package coreElements;

import gameElements.GridCell;
import gameElements.Node;

import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;
import java.util.LinkedList;

/**
 * @author      Christopher Dragert
 * 
 * A non-generic A-Star pathfinder that works with gridcells.
 * 
 */
public class AStar {
	
	private static LinkedList<Node> open;
	private static Vector<Node> closed;
	
	public static Stack<GridCell> getPath(GridCell start, GridCell destination) {
		open = new LinkedList<Node>();
		closed = new Vector<Node>();
		Vector<Node> searchTree = new Vector<Node>();
		Stack<GridCell> victory = null;
		
		Node n = new Node(null, 0, start.distance(destination), start);
		searchTree.add(n);
		open.add(n);
		
		while(!open.isEmpty()) {
			
			//remove the first node from Open
			n = open.removeFirst();
			closed.add(n);
			
			if (n.getGridCell() == destination) {
				victory = new Stack<GridCell>();
				victory.push(n.getGridCell());
				while (n.getParent() != null) {
					n = n.getParent();
					victory.push(n.getGridCell());
				}
			}
			else {
				//expand the node
				Vector<GridCell> neighbours = Game.getInstance().getMaze().get4Neighbours(n.getGridCell(), null);
				for (Iterator<GridCell> i = neighbours.iterator(); i.hasNext();){
					GridCell candidate = i.next();
					if (!isContained(candidate) && candidate.inMaze()) {
						double g = n.getG() + n.getGridCell().distance(candidate);
						double h = candidate.distance(destination);
						Node n_new = new Node(n, g, h, candidate);
						if (h<0) h = 0;
						n.addChild(n_new);
						insert(n_new);
					}
				}
			}
		}
		return victory;
	}
	
	private static boolean isContained(GridCell gc) {
		for (Iterator<Node> it = open.iterator();it.hasNext();){
			if (it.next().getGridCell() == gc) {
				return true;
			}
		}
		for (Iterator<Node> it = closed.iterator();it.hasNext();){
			if (it.next().getGridCell() == gc) {
				return true;
			}
		}
		return false;
	}
	
	private static void insert(Node n_new) {
		int index = 0;
		boolean added = false;
		for (Iterator<Node> it = open.iterator();it.hasNext();) {
			Node n = it.next();
			if (n_new.getG() + n_new.getH() <= n.getG() + n.getH()) {
				open.add(index, n_new);
				added = true;
				break;
			}
			index++;
		}
		if (!added) {
			open.addLast(n_new);
		}
	}
}
