package coreElements;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.*;


/**
 * @author      Christopher Dragert
 * 
 * The menu GUI to change game settings.
 * 
 */
public class MenuPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	//properties manager
	PropertiesManager pm;
	FBMain app;
	
	//components
	JButton okButton;
	LabelledJSlider modeSlider;
	LabelledJSlider FPSSlider;
	JCheckBox braidBox;
	JCheckBox velocityBox;
	LabelledJSlider racerSlider;
	JCheckBox respawnBox;
	LabelledJSlider blockersSlider;
	JCheckBox blockerBox;
	LabelledJSlider sizeSlider;
	LabelledJSlider densitySlider;
	LabelledJSlider braidSlider;
	
	
	public MenuPanel(PropertiesManager pm, FBMain app) {
		
		this.pm = pm;
		this.app = app;
		
		this.setSize(Game.WIDTH + 70, Game.HEIGHT);
		this.setPreferredSize(new Dimension(Game.WIDTH + 54, Game.HEIGHT + 52));
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		//general settings layout
		
		JPanel generalPanel = new JPanel();
		generalPanel.setMaximumSize(new Dimension(500,200));
		generalPanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("General Settings"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		generalPanel.setLayout(new SpringLayout());
		
		//layout menu
		JLabel modeLabel = new JLabel("Mode", JLabel.TRAILING);
		generalPanel.add(modeLabel);
		modeSlider = new LabelledJSlider(1,3,1);
		modeLabel.setLabelFor(modeSlider);
		generalPanel.add(modeSlider);
		
		JLabel desiredFPSLabel = new JLabel("Max FPS", JLabel.TRAILING);
		generalPanel.add(desiredFPSLabel);
		FPSSlider = new LabelledJSlider(0, 100, 10);
		desiredFPSLabel.setLabelFor(FPSSlider);
		generalPanel.add(FPSSlider);
		
		JLabel showBraidsLabel = new JLabel("Show Maze Braids", JLabel.TRAILING);
		generalPanel.add(showBraidsLabel);
		braidBox = new JCheckBox();
		showBraidsLabel.setLabelFor(braidBox);
		generalPanel.add(braidBox);
		
		JLabel showVelocityLabel = new JLabel("Show Velocity", JLabel.TRAILING);
		generalPanel.add(showVelocityLabel);
		velocityBox = new JCheckBox();
		showVelocityLabel.setLabelFor(velocityBox);
		generalPanel.add(velocityBox);
		
		SpringUtilities.makeCompactGrid(generalPanel,
                4, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad
		
		this.add(generalPanel);
		
		//game settings
		
		JPanel gamePanel = new JPanel();
		gamePanel.setMaximumSize(new Dimension(500,200));
		gamePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Game Settings"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		gamePanel.setLayout(new SpringLayout());
		
		JLabel numRacersLabel = new JLabel("Number of Racers", JLabel.TRAILING);
		gamePanel.add(numRacersLabel);
		racerSlider = new LabelledJSlider(0, 500, 50);
		numRacersLabel.setLabelFor(racerSlider);
		gamePanel.add(racerSlider);
	
		JLabel racerRespawnLabel = new JLabel("Racers Respawn?", JLabel.TRAILING);
		gamePanel.add(racerRespawnLabel);
		respawnBox = new JCheckBox();
		racerRespawnLabel.setLabelFor(respawnBox);
		gamePanel.add(respawnBox);
		
		JLabel numBlockersLabel = new JLabel("Number of Blockers", JLabel.TRAILING);
		gamePanel.add(numBlockersLabel);
		blockersSlider = new LabelledJSlider(0, 500, 50);
		numBlockersLabel.setLabelFor(blockersSlider);
		gamePanel.add(blockersSlider);
		
		JLabel blockersMoveLabel = new JLabel("Blockers Move?", JLabel.TRAILING);
		gamePanel.add(blockersMoveLabel);
		blockerBox = new JCheckBox();
		blockersMoveLabel.setLabelFor(blockerBox);
		gamePanel.add(blockerBox);
		
		SpringUtilities.makeCompactGrid(gamePanel,
                4, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad
		
		this.add(gamePanel);
		
		//maze settings layout
		JPanel mazePanel = new JPanel();
		mazePanel.setMaximumSize(new Dimension(500, 175));
		mazePanel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Maze Settings"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		mazePanel.setLayout(new SpringLayout());
		
		JLabel sizeLabel = new JLabel("Size", JLabel.TRAILING);
		mazePanel.add(sizeLabel);
		sizeSlider = new LabelledJSlider(5, 40, 5);
		sizeLabel.setLabelFor(sizeSlider);
		mazePanel.add(sizeSlider);
		
		JLabel densityLabel = new JLabel("Density", JLabel.TRAILING);
		mazePanel.add(densityLabel);
		densitySlider = new LabelledJSlider(1, 51, 10);
		densityLabel.setLabelFor(densitySlider);
		mazePanel.add(densitySlider);
		
		JLabel braidsLabel = new JLabel("Braids", JLabel.TRAILING);
		mazePanel.add(braidsLabel);
		braidSlider = new LabelledJSlider(0, 25, 5);
		braidsLabel.setLabelFor(braidSlider);
		mazePanel.add(braidSlider);
		
		SpringUtilities.makeCompactGrid(mazePanel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad
		
		this.add(mazePanel);
		
		okButton = new JButton("Start!");
		okButton.setVisible(true);
		okButton.setMaximumSize(new Dimension(75,30));
		okButton.addActionListener(new ActionListener() {
			 
            public void actionPerformed(ActionEvent e)
            {
                //Execute when button is pressed
                setValues();
                startNewGame();
            }
        });      
		this.add(okButton);
		
		readValues();
	}
	
	private void startNewGame() {
		app.startNewGame();
	}
	

	//properties
	public void readValues() {
		
		Properties props = pm.getProperties();
		
		modeSlider.setValue(Integer.parseInt(props.getProperty("part"))) ;
		FPSSlider.setValue(Integer.parseInt(props.getProperty("FPS")));
		braidBox.setSelected(Boolean.parseBoolean(props.getProperty("show_braids")));
		velocityBox.setSelected(Boolean.parseBoolean(props.getProperty("show_velocity")));
		racerSlider.setValue(Integer.parseInt(props.getProperty("racers")));;
		respawnBox.setSelected(Boolean.parseBoolean(props.getProperty("racerRespawn")));
		blockersSlider.setValue(Integer.parseInt(props.getProperty("blockers")));
		blockerBox.setSelected(Boolean.parseBoolean(props.getProperty("blockersMove")));
		sizeSlider.setValue(Integer.parseInt(props.getProperty("n")));
		densitySlider.setValue(Integer.parseInt(props.getProperty("density")));
		braidSlider.setValue(Integer.parseInt(props.getProperty("braids")));
	}
	
	private void setValues() {
		Properties props = pm.getProperties();
		props.setProperty("part", ((Integer)modeSlider.getValue()).toString());
		props.setProperty("FPS", ((Integer)FPSSlider.getValue()).toString());
		props.setProperty("show_braids", ((Boolean)braidBox.isSelected()).toString());
		props.setProperty("show_velocity", ((Boolean)velocityBox.isSelected()).toString());
		props.setProperty("racers", ((Integer)racerSlider.getValue()).toString());
		props.setProperty("racerRespawn", ((Boolean)respawnBox.isSelected()).toString());
		props.setProperty("blockers", ((Integer)blockersSlider.getValue()).toString());
		props.setProperty("blockersMove", ((Boolean)blockerBox.isSelected()).toString());
		props.setProperty("n", ((Integer)sizeSlider.getValue()).toString());
		props.setProperty("density", ((Integer)densitySlider.getValue()).toString());
		props.setProperty("braids", ((Integer)braidSlider.getValue()).toString());
		
		pm.setProperties(props);
	}

	private class LabelledJSlider extends JSlider {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 1272264978298134989L;

		public LabelledJSlider(int min, int max, int tickSpace) {
			super(min, max);
			setMajorTickSpacing(tickSpace);
			setPaintTicks(true);
			setPaintLabels(true);
			Font font = new Font("Serif", Font.PLAIN, 10);
			setFont(font);
			setMaximumSize(new Dimension(300,100));
			setSize(new Dimension(300,100));
		}
	}
	
}
