package coreElements;

/**
 * @author      Christopher Dragert
 * 
 * Creates 2d vectors, handles all related calculations
 * 
 */
public class Vector2d {
	public double x;
	public double y;
	
	public Vector2d(double x, double y){
		this.x = x;
		this.y = y;
	}

	public Vector2d getUnit() {
		double magnitude = this.magnitude();
		return new Vector2d(x/magnitude, y/magnitude);
	}
	
	public void set(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	public void add(Vector2d v) {
		x += v.x;
		y += v.y;
	}
	
	public double magnitude() {
		return Math.sqrt(x * x + y * y);
	}
	
	public void zero() {
		x = 0.0;
		y = 0.0;
	}
	
	public Vector2d projectOnto(Vector2d v) {
		double scale = this.dotProduct(v)/ (v.x*v.x + v.y*v.y);
		return new Vector2d(scale*v.x , scale*v.y);
	}
	
	public double dotProduct(Vector2d v) {
		return x*v.x + y*v.y;
	}
	
	public void scale(double s) {
		x = x * s;
		y = y * s;
	}
	
	public double distance(Vector2d v){
		return Math.sqrt((x - v.x)*(x-v.x) + (y - v.y)*(y-v.y));  
	}
		

}

