package gameElements;

import java.util.Stack;

import coreElements.Vector2d;

/**
 * @author      Christopher Dragert
 * 
 * An abstraction of game characters, providing the basics of force-based movement
 * 
 */
public abstract class Character {
	
	//position
	public Vector2d p;
	
	//velocity
	public Vector2d v;
	
	//acceleration
	//don't need it - assume m = 1, thus F=a
	//use F where accel is needed.
	
	//the current path of the character
	protected Stack<GridCell> path;
	
	//forces
	protected Vector2d F;
	
	protected boolean newExit;
	
	public abstract void applyGoals();
	
	public abstract void update(double time);
	
	public void addForce(Vector2d vec) {
		F.add(vec);
	}
	
	public void addForce(double x, double y) {
		F.x += x;
		F.y += y;
	}
	
	public void clearForces() {
		F.zero();
	}

	public void notifyNewExit() {
		newExit = true;
	}
}
