package gameElements;

import java.util.Stack;

import coreElements.AStar;
import coreElements.Game;
import coreElements.Vector2d;

/**
 * @author      Christopher Dragert
 * 
 * The primary object in the game.  It is drawn to the exit using force-based pathfinding.
 * 
 */
public class Racer extends Character {
	public static double max_v;
	private static GridCell exit;
	
	private int stuck;
	private GridCell current;

	//the current goal.
	private GridCell goal;

	public Racer(double x, double y, Stack<GridCell> path) {
		p = new Vector2d(x,y);
		v = new Vector2d(0,0);
		F = new Vector2d(0,0);

		this.path = path;
		if (this.path != null) {
			goal = this.path.pop();
		}
		else {
			this.path = new Stack<GridCell>();
			goal = Game.getInstance().getMaze().getGridLocation(p);
		}
		stuck = 0;
		newExit = false;
	}
	
	
	public void update(double t){

		GridCell location = Game.getInstance().getMaze().getGridLocation(p.x, p.y);
		
		if (newExit) {
			path.clear();
			newExit = false;
		}
		
		if (location.isExit()) {
			Game.getInstance().remove(this);
		}
		
		else {
			if (atGoal() && !path.empty()) {
				//continue on the path
				setGoal(path.pop());
			}

			else if (Game.getInstance().getMaze().getGridLocation(p.x, p.y) == current){
				stuck++;
				if (stuck == 75 || path.empty()) {
					path = AStar.getPath(current, exit);
					try {
						goal = path.pop();
					}
					catch (Exception e) {
						System.out.print("Exception! This is caused by pathfinding when a racer is in the water.");
						System.exit(-1);
					}
				}
			}
			else {
				stuck = 0;
				current = Game.getInstance().getMaze().getGridLocation(p.x, p.y);
			}
			
			F.scale(0.5);
			
			v.x += F.x*t;
			v.y += F.y*t;
			
			if (v.magnitude() > max_v) {
				v = v.getUnit();
				v.scale(max_v);
			}
			
			double oldX = p.x;
			double oldY = p.y;
			
			p.x += v.x * t;
			//p.x += v.x * t + F.x/2*t*t;
			if (p.x <= 0) p.x = 1;
			if (p.x >= Game.WIDTH) p.x = Game.WIDTH-1;
			p.y += v.y * t;
			//p.y += v.y * t + F.y/2*t*t;
			if (p.y <= 0) p.y =1;
			if (p.y >= Game.HEIGHT) p.y = Game.HEIGHT-1;
			
			if (!Game.getInstance().getMaze().getGridLocation(p.x, p.y).inMaze()) {
				p.x = p.x - 2 * (p.x-oldX);
				p.y = p.y - 2 * (p.y-oldY);
			}
		}
	}
	


	public void setGoal(GridCell g) {
		goal = g;
	}
	
	@Override
	public void applyGoals() {	
		//have the goal attract the racer
		Vector2d exitForceA = new Vector2d(goal.guiX+Game.x_scale/2 - p.x,
				goal.guiY + Game.x_scale/2 - p.y);
		exitForceA = exitForceA.getUnit();
		exitForceA.scale(.75);
		addForce(exitForceA);
	}
	
	/*
	 * Returns true if the racer is in the grid-cell that is its current goal.
	 */
	private boolean atGoal() {
		if (goal==Game.getInstance().getMaze().getGridLocation(p)) {
			return true;
		}
		return false;
	}


	public static void setExit(GridCell exit) {
		Racer.exit = exit;
	}
}
